package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.rulematch.containerbind.checkqty;

import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesContainerPartsModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackage;
import cn.estsh.i3plus.pojo.mes.bean.MesContainerPackageDetail;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Component
public class MesContainerBindNotMixService extends MesAbstractContainerBindCheckQtyService {
    public MesContainerBindNotMixService() {
        super(MesExtEnumUtil.CONTAINER_TYPE_MIX_TYPE.MIX_TYPE_10);
    }

    @Override
    public Map<String, Object> doCheck(String organizeCode, MesContainerPackage containerPackage, List<MesContainerPackageDetail> packageDetails, List<MesContainerPartsModel> containerParts) {
        Map<String, Object> result = new HashMap<>();
        result.put(MesPcnExtConstWords.RESULT, false);

        final Set<String> partNoSet = packageDetails.stream().map(MesContainerPackageDetail::getPartNo).collect(Collectors.toSet());
        if (StringUtils.isEmpty(partNoSet)) {
            result.put(MesPcnExtConstWords.MESSAGE, "绑定的底层条码中不包含零件信息");
            return result;
        }
        if (partNoSet.size() > 1) {
            result.put(MesPcnExtConstWords.MESSAGE, String.format("容器条码【%s】不允许混包！", containerPackage.getContainerSn()));
            return result;
        }
        String partNo = partNoSet.iterator().next();
        for (MesContainerPartsModel containerPart : containerParts) {
            if (containerPart.getQty() == null || !Objects.equals(containerPart.getPartNo(), partNo)) {
                continue;
            }

            if (containerPackage.getQty() > containerPart.getQty()) {
                result.put(MesPcnExtConstWords.MESSAGE, String.format("容器条码【%s】超出了包装规格", containerPackage.getContainerSn()));
                return result;
            }
            if (containerPackage.getQty().equals(containerPart.getQty().intValue())) {
                containerPackage.setPackageStatus(MesExtEnumUtil.CONTAINER_PACKAGE_STATUS.STATUS_20.getValue());
            }
        }

        result.put(MesPcnExtConstWords.RESULT, true);
        return result;
    }

    private String getPartNo(List<MesContainerPackageDetail> packageDetails) {
        for (MesContainerPackageDetail packageDetail : packageDetails) {
            if (!StringUtils.isEmpty(packageDetail.getPartNo())) {
                return packageDetail.getPartNo();
            }
        }
        return "";
    }
}
